@@ -2,7 +2,7 @@
 
 package Git::Hooks::CheckJira;
 # ABSTRACT: Git::Hooks plugin which requires citation of JIRA issues in commit messages
-$Git::Hooks::CheckJira::VERSION = '2.8.1';
+$Git::Hooks::CheckJira::VERSION = '2.9.0';
 use 5.010;
 use utf8;
 use strict;
@@ -612,7 +612,7 @@ Git::Hooks::CheckJira - Git::Hooks plugin which requires citation of JIRA issues
 
 =head1 VERSION
 
-version 2.8.1
+version 2.9.0
 
 =head1 SYNOPSIS
 
@@ -620,49 +620,42 @@ As a C<Git::Hooks> plugin you don't use this Perl module directly. Instead, you
 may configure it in a Git configuration file like this:
 
   [githooks]
+
+    # Enable the plugin
     plugin = CheckJira
+
+    # These users are exempt from all checks
     admin = joe molly
 
   [githooks "checkjira"]
+
+    # Configure the URL and the admin credentials to interact with the JIRA
+    # server.
     jiraurl = https://jira.example.net
     jirauser = jiradmin
     jirapass = my-secret
 
+    # Look for JIRA keys at the beginning of the commit messages title, enclosed
+    # in brackets.
     matchlog = (?s)^\\[([^]]+)\\]
-    jql = project IN (ABC, UTF, GIT) AND issuetype IN (Bug, Story) AND status IN ("In progress", "In testing")
+
+    # Impose restrictions on valid JIRA issues
+    jql = project IN (ABC, UTF, GIT) AND \
+          issuetype IN (Bug, Story) AND \
+          status IN ("In progress", "In testing")
+
+    # Require that all cited JIRA issues be assigned to the user pushing the
+    # commits.
     by-assignee = true
+
+    # Commits pushed to master must cite JIRAs associated with the fixVersion
+    # 'future'
     fixversion = refs/heads/master             future
+
+    # Commits pushed to release branches must cite JIRAs associated with the
+    # fixVersion named after the same major.minor version number.
     fixversion = ^refs/heads/(\\d+\\.\\d+)\\.  ^$+
 
-The first section enables the plugin and defines the users C<joe> and C<molly>
-as administrators, effectivelly exempting them from any restrictions the plugin
-may impose.
-
-The second instance enables C<some> of the options specific to this plugin.
-
-The options C<jiraurl>, C<jirauser>, and C<jirapass> specify the JIRA server and
-are usually kept in the global Git configuration file and not in the
-repository's configuration file since they are sensitive and usually the same
-for all repositories.
-
-The C<matchlog> option requires that the JIRA keys be cited at the beginning of
-the commit messages title, enclosed in brackets. By default JIRA keys can be
-cited anywhere, but it's usually a good idea to restrict where they should be
-cited to avoid falselly identifying other words as JIRA keys.
-
-The C<jql> option imposes more restrictions on the cited JIRA issues (besides
-the few restrictions that are imposed by default, such as they being resolved
-and they exist).
-
-The C<by-assignee> option requires that all cited JIRA issues be assigned to the
-user pushing the commits.
-
-The firt C<fixversion> option requires that commits pushed to C<master> must
-cite JIRA issues associated with the fixVersion C<future>. The second option
-requires that commits pushed to branches named after a version name
-(e.g. C<1.0.1>) must cite JIRA issues associated with a fixVersion named after
-the same C<major.minor> version number (e.g. C<1.0>).
-
 =head1 DESCRIPTION
 
 This L<Git::Hooks> plugin hooks itself to the hooks below to guarantee that
