@@ -2,7 +2,7 @@
 
 package Git::Hooks::PrepareLog;
 # ABSTRACT: Git::Hooks plugin to prepare commit messages before being edited
-$Git::Hooks::PrepareLog::VERSION = '2.8.1';
+$Git::Hooks::PrepareLog::VERSION = '2.9.0';
 use 5.010;
 use utf8;
 use strict;
@@ -142,7 +142,7 @@ Git::Hooks::PrepareLog - Git::Hooks plugin to prepare commit messages before bei
 
 =head1 VERSION
 
-version 2.8.1
+version 2.9.0
 
 =head1 SYNOPSIS
 
@@ -150,23 +150,20 @@ As a C<Git::Hooks> plugin you don't use this Perl module directly. Instead, you
 may configure it in a Git configuration file like this:
 
   [githooks]
+
+    # Enable the plugin
     plugin = PrepareLog
 
   [githooks "preparelog"]
+
+    # Grok issue names from the part of the branch name matching this regular
+    # expression, which matches JIRA issue IDs.
     issue-branch-regex = [A-Z]+-\\d+
+
+    # The grokked issue ID should be inserted as a message trailer, keyed by
+    # "JIRA".
     issue-place = key Jira
 
-The first section enables the plugin.
-
-The second section makes the message include an issue ID grokked by the current
-branch name. If the current branch matches the C<issue-branch-regex> option it's
-name will be used as the issue ID. In this case, it matches a
-L<JIRA|https://www.atlassian.com/software/jira> ID. The C<issue-place> option
-specifies that the JIRA ID should be inserted as a message trailer, keyed by
-"JIRA". For example:
-
-  Jira: PRJ-123
-
 =head1 DESCRIPTION
 
 This L<Git::Hooks> plugin hooks itself to the C<prepare-commit-msg>. It's
